#coding: utf-8
'''
Created on 17.09.2010

@author: akvarats
'''

import threading

from django.conf import settings
from django.utils.importlib import import_module

import logger

class ExtensionException(Exception):
    '''
    Класс исключений для расширений
    '''
    pass

class ExtensionPointDoesNotExist(ExtensionException):
    '''
    Возникает в случае если не найдена точка расширения
    '''
    def __init__(self, extension_name, *args, **kwargs):
        self.extension_name = extension_name

    def __str__(self):
        return 'ExtensionPoint "%s" not founded' % self.extension_name

class ExtensionPoint:
    '''
    Класс, описывающий точку расширения с именем *name* и
    вызываемой по умолчанию функцией *default_listener*
    '''
    def __init__(self, name = '', default_listener = None):

        # название точки расширения
        # названия точек расширения необходимо выполять в форме
        # mis.core.schedules.get-busy-perions
        self.name = name
        # человеческое название точки расширения
        self.verbose_name = u'Точка расширения'
        # листенер, который добавляется по умолчанию
        self.default_listener = default_listener

class ExtensionHandler:
    '''
    Класс-обертка над обработчиком точки расширения *handler*
    с именем *name* и режимом вызова *call_type*
    '''
    #===========================================================================
    # Константы, которые определяет порядок вызова листенера
    # точки расширения
    #===========================================================================
    INSTEAD_OF_PARENT = 0
    BEFORE_PARENT = 1
    AFTER_PARENT = 2

    def __init__(self, handler = None, call_type = INSTEAD_OF_PARENT):
        self.handler = handler
        self.call_type = call_type

ExtensionListener = ExtensionHandler # совместимость

class ExtensionManager:
    '''
    Класс, который управляет точками расширения приложений
    '''
    __shared_state = dict(
        loaded = False,
        # словарь точек расширения. ключом являются наименование точки ExtensionPoint.name
        extensions = {},
        # словарь листенер, которые выполняются для точек расширения
        # ключом является наименование точки расширения, значениями - список
        listeners = {},
        # стек выполнения листенеров
        stack = {},
        _write_lock = threading.RLock(),
    )

    def __init__(self):
        self.__dict__ = self.__shared_state

    def _populate(self):
        '''
        Метод собирает точки расширения из
        app_meta приложений
        '''
        if self.loaded:
            return False
        self._write_lock.acquire()
        try:
            if self.loaded:
                return False
            for app_name in settings.INSTALLED_APPS:
                try:
                    module = import_module('.app_meta', app_name)
                except ImportError, err:
                    if err.args[0].find('No module named') == -1:
                        raise
                    continue
                proc = getattr(module, 'register_extensions', None)
                if callable(proc):
                    proc()
            self._loaded = True
        finally:
            self._write_lock.release()

    def _validate_extension_point(self, extension_point):
        '''
        Проверяет точку расширения на возможность регистрации
        в менеджере
        '''
        return (extension_point and
                isinstance(extension_point, ExtensionPoint) and
                extension_point.name and
                extension_point.name.strip() and
                isinstance(extension_point.default_listener, ExtensionListener) and
                not self.extensions.has_key(extension_point.name))

    def register_point(self, extension_point):
        '''
        Добавляет точку расширения
        '''
        if not self._validate_extension_point(extension_point):
            return

        point_key = extension_point.name.strip()
        self.extensions[point_key] = extension_point
        self.listeners[point_key] = [extension_point.default_listener,]

    append_point = register_point # для совместимости

    def register_point_external(self, extension_point):
        '''
        Метод регистрации точки расширения, который должен использоваться
        извне.

        Данный метод отличается от register_point тем, что при его
        использовании выставляются внутренние локи, что потенциально
        может привести к падению производительности системы.

        Поэтому, если не лень, используйте не декоратор @extension_point,
        а пишите определение точки расширения в app_meta. Да прибудет с вами
        сила, чтоле.
        '''
        if not self._validate_extension_point(extension_point):
            return

        self._write_lock.acquire()
        try:
            self.register_point(extension_point)
        except:
            logger.exception(u'Не удалось зарегистрировать точку расширения \'%s\'' % extension_point.name)
        finally:
            self._write_lock.release()

    def check_point_exists(self, extension_point_name):
        '''
        Проверяет, существует ли во внутреннем кеше определение
        точки расширения с указанным именем
        '''
        return self.extensions.has_key(extension_point_name)

    def register_handler(self, extension_name, listener):
        '''
        Добавляет листенер точки расширения с именем extension_name
        '''
        if (not listener or
            not isinstance(listener, ExtensionListener) or
            not listener.handler):
                # передали неправильное определение листенера
                # ничего не делаем
                return
        try:
            self.listeners[extension_name].append(listener)
        except KeyError:
            raise ExtensionPointDoesNotExist(extension_name=extension_name)

    append_listener = register_handler # для совместимости

    def execute(self, extension_name, *args, **kwargs):
        '''
        Выполняет связанное с точкой расширения текущее действие
        '''
        result = None
        if not self.loaded:
            self._populate()

        if not self.extensions.has_key(extension_name) or not self.listeners.has_key(extension_name):
            return None

        if  not self.stack.has_key(extension_name) or not self.stack[extension_name]:
            # необходимо выполнить подготовку стека вызовов
            listener_stack = []
            if len(self.listeners[extension_name]) == 1 and not self.listeners[extension_name]:
                # обработка случая, когда в качестве дефолтного листенера задан
                # пустой обработчик и больше обработчиков нет
                listener_stack = [None,]
            else:
                for listener in self.listeners[extension_name]:
                    if not listener or not listener.handler:
                        continue
                    if listener.call_type == ExtensionListener.INSTEAD_OF_PARENT:
                        listener_stack = [listener,]
                    elif listener.call_type == ExtensionListener.BEFORE_PARENT:
                        listener_stack.insert(0, listener)
                    else:
                        listener_stack.append(listener)

            self.stack[extension_name] = listener_stack

        # собственно, выполняем точки расширения
        for listener in self.stack[extension_name]:
            kwargs['ext_result'] = result
            if listener and callable(listener.handler):
                result = listener.handler(*args, **kwargs)

        return result

    def get_handlers(self, extension_name):
        '''
        Возвращает список хендлеров, которые объявлены для указанной точки расширения.

        Хендлеры возвращаются списком. Элементы данного списка идут в порядке
        их регистрации
        '''
        self._populate()
        return self.listeners.get(extension_name, [])


#===============================================================================
# Декораторы для работы с точками расширения
#===============================================================================
def extension_point(name=''):
    '''
    Декортатор, с помощью которого определяется точка расширения c именем *name*.

    Данный декоратор должен использоваться над дефолтным хендлером
    точки расширения с указанным именем name
    '''
    def inner(f):
        def wrapper(*args, **kwargs):
            return ExtensionManager().execute(name, *args, **kwargs)
        # формируем определение точки расширения
        if not ExtensionManager().check_point_exists(name):
            # пытаемся добавить точку расширения
            ExtensionManager().register_point_external(ExtensionPoint(name=name,default_listener=ExtensionHandler(f)))
        return wrapper
    return inner



# coding:utf-8

import datetime

from django.db import models, connection, transaction, router, connections
from django.db.models.query import QuerySet
from django.db.models.deletion import Collector

from m3 import json_encode


def safe_delete(model):
    """
    Функция выполняющая "безопасное" удаление записи из БД.
    В случае, если удаление не удалось по причине нарушения целостности,
    то возвращается false. Иначе, true
    к тому же функция пересчитывает MPTT индексы дерева
    т.к. стандартный пересчет запускается при вызове model_instance.delete()
    """
    models.signals.pre_delete.send(sender=model.__class__, instance=model)
    try:
        cursor = connection.cursor()  # @UndefinedVariable
        sql = "DELETE FROM %s WHERE id = %s" % (connection.ops.quote_name(
            model._meta.db_table), model.id)  # @UndefinedVariable
        cursor.execute(sql)
        transaction.commit_unless_managed()
    except Exception as e:
        # Встроенный в Django IntegrityError не генерируется.
        # Кидаются исключения, специфичные для каждого драйвера БД.
        # Но по спецификации PEP 249 все они называются IntegrityError
        if e.__class__.__name__ == 'IntegrityError':
            return False
        raise

    # добавим пересчет mptt дерева
    # (т.к. стандартный пересчет вешается на метод self.delete()
    if hasattr(model, '_tree_manager') and callable(
            getattr(model._tree_manager, '_close_gap', None)):
        # это видимо mptt моедль
        opts = model._meta
        tree_width = (getattr(model, opts.right_attr) -
                      getattr(model, opts.left_attr) + 1)
        target_right = getattr(model, opts.right_attr)
        tree_id = getattr(model, opts.tree_id_attr)
        model._tree_manager._close_gap(tree_width, target_right, tree_id)

    models.signals.post_delete.send(sender=model.__class__, instance=model)
    return True


def queryset_limiter(queryset, start=0, limit=0):
    """
    "Вырезает" из QuerySet'a записи начиная с позиции start
    до записи start+limit.
    Возвращает (rows, total, ), где
    rows -  QuerySet с вырезанными записями
    total - общее кол-во записей в queryset'e
    """

    assert (
        isinstance(queryset, QuerySet) or getattr(queryset, '__iter__')
    ), (
        'queryset must be either instance of '
        'django.db.models.query.QuerySet or iterable'
    )

    if start < 0:
        start = 0
    if limit < 0:
        limit = 0
    total = queryset.count() if isinstance(
        queryset, QuerySet) else len(queryset)
    rows = queryset[start:start + limit]
    return rows, total


class BaseEnumerate(object):
    """
    Базовый класс для создания перечислений.
    """
    # В словаре values описываются перечисляемые константы
    # и их человеческое название
    # Например: {STATE1: u'Состояние 1', CLOSED: u'Закрыто'}
    values = {}

    @classmethod
    def get_choices(cls):
        """
        Используется для ограничения полей ORM и в качестве источника данных
        в ArrayStore и DataStore ExtJS
        """
        return cls.values.items()

    get_items = get_choices

    @classmethod
    def get_constant_value_by_name(cls, name):
        """
        Возвращает значение атрибута константы, которая используется в
        качестве ключа к словарю values
        """
        if not isinstance(name, basestring):
            raise TypeError("'name' must be a string")

        if not name:
            raise ValueError("'name' must not be empty")

        return cls.__dict__[name]


class BaseObjectModel(models.Model):
    """
    Базовая модель для объектов системы.
    Сюда будут добавляться общие свойства и методы,
    которые могут быть перекрыты в дальнейшем
    """
    @json_encode
    def display(self):
        """
        Отображение объекта по-умолчанию. Отличается от __unicode__ тем,
        что вызывается при json сериализации в m3.core.json.M3JSONEncoder
        """
        return unicode(self)

    def __unicode__(self):
        """ Определяет текстовое представление объекта """
        name = getattr(self, 'name', None) or getattr(self, 'fullname', None)
        if name:
            if callable(name):
                name = name()
            return u'{%s: %s}' % (self.pk, name)
        else:
            return u'{%s}' % self.pk

    @classmethod
    def get_verbose_name(cls):
        return cls._meta.verbose_name

    def safe_delete(self):
        """
        Функция выполняющая "безопасное" удаление записи из БД.
        В случае, если запись не удалось удалисть по причине нарушения
        целостности, возвращается False, иначе True.
        """
        return safe_delete(self)

    def get_related_objects(self, using=None):
        """
        Возвращает структуру содержащую классы моделей,
        первичные ключи и экземпляры записей, зависящие от текущей записи.
        Возвращаемая структура имеет вид:
        [(
            КлассМодели1, {
                id1: ЭкземплярМодели1cID1, id2: ЭкземплярМодели1cID2, ...}),
         (
            КлассМодели2, {
                id1: ЭкземплярМодели2cID1, id2: ЭкземплярМодели2cID2, ...} },
         ...
        ]
        @deprecated: Вытаскивает много данных. Сервер может зависнуть!
        """
        using = using or router.db_for_write(self.__class__, instance=self)
        collector = Collector(using=using)
        collector.collect([self])
        return collector.data.items()

    def delete_related(self, affected=None, using=None):
        """
        Стандартное каскадное удаление объектов в django,
        дополненное проверкой на удаляемые классы моделей affected.
        По умолчанию affected содержит пустой
        список - это ограничивает удаляемые модели только текущим классом.
        К перечисленным в affected классам текущий добавляется автоматически.
        Если удаление не удалось выполнить, возвращает False, иначе True.
        Пример: Model1.objects.get(id=1).delete_related(
            affected=[Model2, Model3])
        """
        # Кописаст из django.db.models.Model delete()
        using = using or router.db_for_write(self.__class__, instance=self)
        assert (
            self._get_pk_val() is not None
        ), (
            "%s object can't be deleted because its %s "
            "attribute is set to None."
        ) % (
            self._meta.object_name,
            self._meta.pk.attname
        )

        collector = Collector(using=using)
        collector.collect([self])
        # cut

        affected = affected or []
        assert isinstance(
            affected, list), 'Affected models must be the list type'
        affected.append(self.__class__)

        for model in collector.data.keys():
            if model not in affected:
                return False

        collector.delete()
        return True

    class Meta:
        abstract = True


class ForUpdateQuerySet(QuerySet):
    def for_update(self):
        if 'sqlite' in connections[self.db].settings_dict['ENGINE'].lower():
            # Noop on SQLite since it doesn't support FOR UPDATE
            return self
        sql, params = self.query.get_compiler(self.db).as_sql()
        return self.model._default_manager.raw(
            sql.rstrip() + ' FOR UPDATE', params)


class ForUpdateManager(models.Manager):
    def get_query_set(self):
        return ForUpdateQuerySet(self.model, using=self._db)


class ConcurrentEditError(Exception):
    """
    Исключение возникающее при попытке сохранения записи,
    которая была изменена после ее чтения.
    """
    pass


class BaseObjectModelWVersion(BaseObjectModel):
    """
    Базовый класс для версионных записей.
    Нужен для реализации оптимистичной обработки блокировки
    """

    objects = ForUpdateManager()

    version = models.IntegerField(u'Версия записи', default=0)

    def do_lock(self):
        if self.id:
            # блокируем запись с нашей версией от изменения
            q = self.__class__.objects.filter(
                id=self.id, version=self.version).for_update()
            # если удачно блокировали,
            # то можем делать с ней что угодно в рамках транзакции
            if len(list(q)) == 1:
                return True
            else:
                # если блокировать нечего, то значит кто-то ее поменял
                return False
        else:
            return True

    def save(self, *a, **k):
        """
        При каждом сохранении номер версии увеличивается.
        Это нужно чтобы проверять,
        была ли модифицирована запись после последнего получения
        """
        if self.id:
            self.version += 1
        super(BaseObjectModelWVersion, self).save()

    class Meta:
        abstract = True


class ObjectState(BaseEnumerate):
    """
    Состояние объекта
    Используется для определения логики использования записи:
    - если запись "Действует", значит нет ограничений на ее использование
    - если запись "Закрыта",
      значит ее нельзя использовать в новых документах,
      но можно выводить в отчетах и уже существующих данных
    - если запись "Черновик",
      значит ее нельзя использовать в логике приложения и в отчетах.
      По сути, это означает что запись введена не полностью и не утверждена
    """

    VALID = 0
    CLOSED = 1
    DRAFT = 2
    values = {VALID: u'Действует', CLOSED: u'Закрыта', DRAFT: u'Черновик'}


class ObjectManager(models.Manager):
    """
    Менеджер запросов к записям справочника
    Фильтрует записи по периоду действия и состоянию
    """

    def get_default_state(self):
        """
        Для прикладного переопределения состояний, выбираемых по-умолчанию
        """
        return [ObjectState.VALID]

    def __init__(self, date=None, state=None, *a, **kw):
        super(ObjectManager, self).__init__(*a, **kw)
        self.query_on_date = date
        if state:
            if isinstance(state, type([])):
                self.query_state = state
            else:
                self.query_state = [state]
        else:
            self.query_state = self.get_default_state()

    def get_query_set(self):
        # если указывали дату, то отфильтруем на дату,
        # иначе только по состоянию
        if self.query_on_date:
            kwargs = dict(
                begin__lte=self.query_on_date,
                end__gt=self.query_on_date,
                state__in=self.query_state)
        else:
            kwargs = dict(state__in=self.query_state)
        return super(ObjectManager, self).get_query_set().filter(**kwargs)


class BaseObjectModelWState(BaseObjectModel):

    """
    Базовый класс для всех моделей состоянием и периодом действия
    """

    state = models.SmallIntegerField(
        u'Состояние',
        choices=ObjectState.get_choices(),
        default=ObjectState.DRAFT
    )
    begin = models.DateTimeField(
        u'Начало действия',
        null=True,
        blank=True,
        db_index=True,
        default=datetime.date.min
    )
    end = models.DateTimeField(
        u'Окончание действия',
        null=True,
        blank=True,
        db_index=True,
        default=datetime.date.max
    )

    @classmethod
    def get_objects_on_date(cls, date=None):
        """
        Получает менеджер с параметрами.
        Можно писать так: Model.objects_on_date(datetime.today).filter....
        """
        manager = ObjectManager(date=date)
        manager.model = cls
        return manager

    objects_on_date = get_objects_on_date

    class Meta:
        abstract = True



# coding: utf-8

import logging.handlers
import os
import sys
import traceback

from django.http import HttpResponseServerError, HttpRequest
from django.template.loader import render_to_string
from django.core.mail import get_connection
from django.core.mail.message import EmailMessage
from django.utils.html import linebreaks
from django.utils.encoding import force_unicode

__all__ = ['init_logging', 'catch_error_500',
           'info', 'error', 'debug', 'warning']


def init_logging(logs_path):
    '''
    Инициализация питоновского логирования. Срабатывает только один раз.
    Каждый error level записывается в свой лог.
    Нужно добавлять в settings.py или manage.py
    '''
    if hasattr(logging, "m3_set_up_done"):
        return

    if not os.path.exists(logs_path):
        os.mkdir(logs_path)

    loggers = [
        ('root', logging.NOTSET, 'root.log'),
        ('error_logger', logging.ERROR, 'error.log'),
        ('info_logger', logging.INFO, 'info.log'),
        ('debug_logger', logging.DEBUG, 'debug.log'),
        ('warning_logger', logging.DEBUG, 'warning.log')
    ]
    formatter = logging.Formatter("[%(asctime)s] %(message)s")
    formatter.datefmt = '%Y-%m-%d %H:%M:%S'
    for lname, level, fname in loggers:
        t = logging.getLogger(lname)
        t.setLevel(level)
        handler = logging.handlers.TimedRotatingFileHandler(
            os.path.join(logs_path, fname), when='D', encoding='utf-8')
        handler.setFormatter(formatter)
        t.addHandler(handler)

    logging.m3_set_up_done = True


def get_session_info(request):
    '''
    Возвращает строку для лога с информацией о запросе
    '''
    if (request is None) or (not isinstance(request, HttpRequest)):
        return ''
    # Адрес запроса
    result = 'URL: ' + request.get_full_path() + ' (' + request.method + ')'
    # Юзер
    if hasattr(request, 'user'):
        user = request.user
        if user and user.is_authenticated():
            result += ' - ' + user.email + ', ' + user.get_full_name()
    return result + '. '

#====================== ФУНКЦИИ АНАЛОГИЧНЫЕ LOGGING =====================
# Если передать именованный аргумент request = ... , то к сообщению будет
# добавлена информация о запросе
#========================================================================


def info(msg, *args, **kwargs):
    log = logging.getLogger('info_logger')
    msg = get_session_info(kwargs.get('request', None)) + msg
    log.info(msg)

    # Отправка на почту
    send_mail_log(msg, 'INFO')


def error(msg, *args, **kwargs):
    log = logging.getLogger('error_logger')
    msg = get_session_info(kwargs.get('request', None)) + msg
    log.error(msg)

    # Отправка на почту
    send_mail_log(msg, 'ERROR')


def debug(msg, *args, **kwargs):
    '''
    Выводит информацию в debug-log только в случае включенного режима отладки
    '''
    from django.conf import settings

    if settings.DEBUG:
        log = logging.getLogger('debug_logger')
        msg = get_session_info(kwargs.get('request', None)) + msg
        log.debug(msg)


def exception(msg='', *args, **kwargs):
    # Не желаемые ключи логирования
    black_list = [
        'win',  # Запрос браузера
        'request',  # Запрос в БД (sql)
        'query'
    ]

    log = logging.getLogger('error_logger')
    msg = get_session_info(kwargs.get(
        'request', None)) + 'Message: ' + msg + '\n'

    # FIXME: Отрефаторить нахер!!
    exception_info = sys.exc_info()
    try:
        e_type, e_value, e_traceback = exception_info
        e_vars = e_traceback.tb_frame.f_locals
        res = [u'Variables:\n']

        def append(key, val, rjust):
            res.append(
                u'%s: %s\n'.rjust(rjust) % (
                    key,
                    (force_unicode(val)
                        if isinstance(val, basestring) else val)))

        if e_traceback.tb_frame.f_code.co_name != '<module>':
            for key, val in e_vars.items():
                if key in black_list:
                    continue

                append(key, val, rjust=6)
                if hasattr(val, '__dict__'):
                    for obj_key, obj_val in val.__dict__.items():
                        if obj_key[0] != '_' and obj_key not in black_list:
                            append(obj_key, obj_val, rjust=12)
        else:
            res = []

        e_vars_str = ''.join(res)
        tb = traceback.format_exception(e_type, e_value, e_traceback)

        err_message = u'\n %s %s %s' % (
            force_unicode(msg),
            force_unicode(u''.join(tb)),
            force_unicode(e_vars_str)
        )
        log.error(err_message)

        # Отправка на почту
        send_mail_log(err_message, e_type.__name__)

    except:
        _old_good_exception(msg, exception_info, *args, **kwargs)


def _old_good_exception(msg, exception_info, *args, **kwargs):
    """
    Старый добрый способ вывода ошибки, без переменных, зато надежный.
      msg - гневное сообщение разработчикам
      exception_info - информация о оригинальной ошибке
    """
    log = logging.getLogger('error_logger')
    try:
        tb = traceback.format_exception(*exception_info)
        log.error(
            force_unicode(get_session_info(kwargs.get('request')) + msg) +
            '\n' + u''.join(tb)
        )
    except:
        try:
            log.error(force_unicode(msg) + '\n')
        except:
            log.error(u'Некорректная работа логгера')


def warning(msg, *args, **kwargs):
    log = logging.getLogger('warning_logger')
    msg = get_session_info(kwargs.get('request', None)) + msg
    log.warning(msg)

    send_mail_log(msg, 'WARN')


def catch_error_500(request, *args, **kwargs):
    '''
    Функция для перехвата ошибок сервера в боевом режиме.
    Возвращает страницу 500 определенную пользователем.
    '''
    exception(get_session_info(request), exc_info=True)
    return HttpResponseServerError(render_to_string("500.html"))

#=========================================================================


def send_mail_log(msg, err_name='', level=''):
    '''
    Дополнительное письмо с логом на почту администратора
    '''
    from django.conf import settings

    emails_admin = getattr(settings, 'EMAIL_ERRORLOG_ADMIN', False)
    email_from = getattr(settings, 'EMAIL_ADDRESS_FROM', "")

    if emails_admin and email_from:
        try:
            if isinstance(emails_admin, list):
                email_list = [emails_admin, ]
            elif isinstance(emails_admin, basestring):
                email_list = emails_admin.split(',')

            conn = get_connection(
                auth_user=settings.EMAIL_HOST_USER,
                auth_password=settings.EMAIL_HOST_PASSWORD
            )

            # uname есть только по nix системами,
            # на остальных смотрим переменную
            # среды os.getenv('USERNAME')
            # 2-ой параметр - имя системы
            uname = os.uname()[1] if hasattr(os, 'uname') and callable(
                os.uname) else os.getenv('USERNAME')

            msg = linebreaks(msg)
            d = {'body': msg, 'from_email': email_from, 'to': email_list,
                 'subject': u'Логер - %s - %s %s' % (uname, level, err_name)}
            message = EmailMessage(**d)
            message.content_subtype = "html"
            # TODO: на этом моменте сильно тормозит,
            # если есть проблемы с доставкой письма
            # типа нет связи с почтовым сервером
            # или неверный логин и(или) пароль
            # с другой стороны, ускорение отработки доставки писем
            # (в отдельном потоке) приводит к отказу
            # на почтовом сервере
            # (слишком частые !неудачные! попытки соединения)
            # и он вообще все блочит!
            conn.send_messages([message, ])
        except:
            log = logging.getLogger('error_logger')
            msg = 'Error sending e-mail\n'
            log.error(msg)



#coding: utf-8
import os
from setuptools import setup, find_packages

def read(fname):
    try:
        return open(os.path.join(os.path.dirname(__file__),
            fname)).read()
    except IOError:
        return ''

setup(name='m3-legacy',
      version='2.0.0',
      url='https://bitbucket.org/barsgroup/m3-legacy',
      license='MIT',
      author='BARS Group',
      author_email='bars@bars-open.ru',
      package_dir={'': 'src'},
      packages=find_packages('src'),
      description=read('DESCRIPTION'),
      install_requires=read('REQUIREMENTS'),
      long_description=read('README'),
      include_package_data=True,
      classifiers=[
        'Intended Audience :: Developers',
        'Environment :: Web Environment',
        'Natural Language :: Russian',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'License :: OSI Approved :: MIT License',
        'Development Status :: 5 - Production/Stable',
      ],
)



#!coding: utf-8
"""
Общие хелперы
"""

import re
import warnings
from uuid import uuid4
from django.utils import datetime_safe


def is_valid_email(value):
    if (value is None):
        return
    ptn = re.compile(
        r"(?:^|\s)[-a-z0-9_.]+@(?:[-a-z0-9]+\.)+[a-z]{2,6}(?:\s|$)",
        re.IGNORECASE
    )
    return ptn.match(value)


def normalize(str):
    """
    Конвертирует строку в вид, понятный javascript'у
    """
    return str.replace(
        '\r', '\\r'
    ).replace(
        '\n', '\\n'
    ).replace(
        '"', '\\"'
    ).replace(
        "'", "\\'"
    )


def generate_client_id():
    """
    Генерирует уникальный id для визуального компонента.
    """
    return 'cmp_' + str(uuid4())[0:8]


def get_img_size(src_size, dest_size):
    """
    Возвращает размеры изображения в пропорции с оригиналом исходя из того,
    как направлено изображение (вертикально или горизонтально)
    """
    width, height = dest_size
    src_width, src_height = src_size
    if height >= width:
        return (int(float(width)/height*src_height), src_height)
    return (src_width,  int(float(height)/width*src_width))


def generate_id():
    """
    Генерирует восьмизначный random.
    """
    return str(uuid4())[0:8]


def date2str(date, template='%d.%m.%Y'):
    """
    datetime.strftime глючит с годом < 1900
    типа обходной маневр (взято из django)
    WARNING from django:
    # This library does not support strftime's \"%s\" or \"%y\" format strings.
    # Allowed if there's an even number of \"%\"s because they are escaped.
    """
    return datetime_safe.new_datetime(date).strftime(template)


#==============================================================================
# Общие функции получения объектов из базы данных
#==============================================================================
def get_object_by_id(model, object_id):
    """
    Возвращает объект из базы данных указанного типа и
    указанным идентификатором. В случае, если в object_id
    оказался объект типа model, то он и возвращается.
    """
    warnings.warn(
        u'get_object_by_id устарела!',
        category=DeprecationWarning,
        stacklevel=2,
    )

    result = None

    if isinstance(object_id, (int, str, unicode)):
        try:
            result = model.objects.get(pk=object_id)
        except model.DoesNotExist:
            result = None
    elif isinstance(object_id, model):
        result = object_id

    return result
